/*
 * Copyright (c) 2004-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: EdgeGUI.java,v 1.6 2007/05/19 02:45:25 sshirasu Exp $
 */

package edu.indiana.extreme.xbaya.graph.gui;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Stroke;
import java.awt.event.MouseEvent;
import java.awt.geom.CubicCurve2D;

import edu.indiana.extreme.xbaya.XBayaEngine;
import edu.indiana.extreme.xbaya.graph.ControlEdge;
import edu.indiana.extreme.xbaya.graph.Edge;
import edu.indiana.extreme.xbaya.graph.Port;

/**
 * @author Satoshi Shirasuna
 */
public class EdgeGUI implements GraphPieceGUI {

    /**
     * CONTROL_EDGE_STROKE
     */
    public static final Stroke CONTROL_EDGE_STROKE = new BasicStroke(1.0f,
            BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER, 10.0f,
            new float[] { 5.0f }, 0.0f);

    private static final Color lineColor = Color.black;

    private static final Color pointColor = Color.pink;

    private static final Color selectedPointColor = Color.red;

    private static final int POINT_SIZE = 8;

    private Edge edge;

    private boolean selected = false;

    /**
     * @param edge
     * 
     */
    public EdgeGUI(Edge edge) {
        this.edge = edge;
    }

    /**
     * @see edu.indiana.extreme.xbaya.graph.gui.GraphPieceGUI#mouseClicked(java.awt.event.MouseEvent,
     *      edu.indiana.extreme.xbaya.XBayaEngine)
     */
    public void mouseClicked(MouseEvent event, XBayaEngine engine) {
        // Nothing
    }

    /**
     * @param bool
     */
    protected void setSelectedFlag(boolean bool) {
        this.selected = bool;
    }

    /**
     * @return the middle point of the edge
     */
    protected Point getMiddlePosition() {
        Point point1 = getFromPosition();
        Point point2 = getToPosition();

        Point midPoint = new Point((point1.x + point2.x) / 2,
                (point1.y + point2.y) / 2);
        return midPoint;
    }

    /**
     * @param g
     */
    protected void paint(Graphics2D g) {

        Point point1 = getFromPosition();
        Point point2 = getToPosition();

        g.setColor(lineColor);

        Stroke originalStroke = g.getStroke();
        if (this.edge instanceof ControlEdge) {
            g.setStroke(EdgeGUI.CONTROL_EDGE_STROKE);
        }
        paintLine(point1, point2, g);
        g.setStroke(originalStroke);

        g.setColor(this.selected ? pointColor : selectedPointColor);

        Point midPoint = getMiddlePosition();
        g.fillArc(midPoint.x - POINT_SIZE / 2, midPoint.y - POINT_SIZE / 2,
                POINT_SIZE, POINT_SIZE, 0, 360);
    }

    protected static void paintLine(Point point1, Point point2, Graphics2D g) {
        int d = 100;
        int dist = (int) point1.distance(point2);
        if (dist < d) {
            d = dist;
        }
        CubicCurve2D line = new CubicCurve2D.Double(point1.x, point1.y,
                point1.x + d, point1.y, point2.x - d, point2.y, point2.x,
                point2.y);
        g.draw(line);
    }

    private Point getFromPosition() {
        Port port = this.edge.getFromPort();
        return port.getGUI().getPosition();
    }

    private Point getToPosition() {
        Port port = this.edge.getToPort();
        return port.getGUI().getPosition();
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2004-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
